<?php
/**
 * Base Class and Methods for AD Integration Plug-in
 * @package wordpress
 * @subpackage ADAuthInt
 * @version 0.4a
 */
/*
 * Active Directory Integration Plug-in Class
 *
 * Various functions that were included in the original
 * AD Integration plug-in are defined within this class.
 */
class ADInt_Original_Plugin {
/**
 * ===============================================================
 * = Additional functions to be pulled into the ADAuthInt_Plugin =
 * = class.                                                      =
 * = The majority of these functions were copied directly from   =
 * = the AD Integration plug-in, changing only the appropriate   =
 * = variable names where necessary.                             =
 * ===============================================================
 */

	const TABLE_NAME = 'adauthint';
	
	/**
	 * Wrapper
	 * 
	 * @param $arg1 WP_User or username
	 * @param $arg2 username or password
	 * @param $arg3 passwprd or empty
	 * @return WP_User
	 * @uses ADAuthInt_Plugin::ad_authenticate()
	 * @uses ADAI_LOG_INFO
	 * @uses $wp_version
	 */
	public function authenticate($arg1 = NULL, $arg2 = NULL, $arg3 = NULL) {
		global $wp_version;
		
		$this->_log(ADAI_LOG_INFO,'method authenticate() called');
		
		$this->_log(ADAI_LOG_INFO,'WP version: ' . $wp_version);
		
		if (version_compare($wp_version, '2.8', '>=')) {
			return $this->ad_authenticate($arg1, $arg2, $arg3); 
		} else {
			return $this->ad_authenticate(NULL, $arg1, $arg2);
		}
	} /* Wrapper function */

	/**
	 * If the REMOTE_USER evironment is set, use it as the username.
	 * This assumes that you have externally authenticated the user.
	 * / Options /
	 * @uses ADAuthInt_Plugin::user_account_suffix
	 * @uses ADAuthInt_Plugin::base_dn
	 * @uses ADAuthInt_Plugin::domain_controllers
	 * @uses ADAuthInt_Plugin::bind_user
	 * @uses ADAuthInt_Plugin::bind_user_password
	 * @uses ADAuthInt_Plugin::port
	 * @uses ADAuthInt_Plugin::secure_connection
	 * @uses ADAuthInt_Plugin::max_login_attempts
	 * @uses ADAuthInt_Plugin::notify_user
	 * @uses ADAuthInt_Plugin::notify_admin
	 * @uses ADAuthInt_Plugin::auth_from_ad_group
	 * @uses ADAuthInt_Plugin::user_account_suffix
	 * @uses ADAuthInt_Plugin::auto_user_create
	 *
	 * / System Vars /
	 * @uses ADAuthInt_Plugin::_authenticated
	 * @uses ADAuthInt_Plugin::_adldap
	 * @uses ADAuthInt_Plugin::_loglevel
	 *
	 * / System Constants /
	 * @uses ADAI_LOG_NOTICE
	 * @uses ADAI_LOG_DEBUG
	 * @uses ADAI_LOG_INFO
	 * @uses ADAI_LOG_ERROR
	 * @uses ADAI_LOG_NONE
	 *
	 * / System Functions /
	 * @uses ADAuthInt_Plugin::_log()
	 * @uses ADAuthInt_Plugin::_load_options()
	 * @uses ADAuthInt_Plugin::_get_failed_logins_within_block_time()
	 * @uses ADAuthInt_Plugin::_display_blocking_page()
	 * @uses ADAuthInt_Plugin::_store_failed_login()
	 * @uses ADAuthInt_Plugin::_cleanup_failed_logins()
	 * @uses ADAuthInt_Plugin::_check_authorization_by_group()
	 * @uses ADAuthInt_Plugin::_get_user_role_equiv()
	 * @uses ADAuthInt_Plugin::_get_display_name_from_AD()
	 * @uses ADAuthInt_Plugin::_create_user()
	 * @uses ADAuthInt_Plugin::_update_user()
	 *
	 * / WordPress Functions /
	 * @uses get_userdatabylogin()
	 * @uses username_exists()
	 * @uses WP_User::_construct()
	 */
	public function ad_authenticate( $user = NULL, $username = '', $password = '' ) {
		$user_id = NULL;
		$this->_authenticated = false;
		if( empty( $username ) || empty( $password ) ) {
			$this->_log( ADAI_LOG_DEBUG, 'The username and/or password was empty' );
			return;
		}
		
		// IMPORTANT!
		$username = strtolower( $username );
		
		$this->_log( ADAI_LOG_NOTICE, 'username: ' . $username );
		$this->_log( ADAI_LOG_DEBUG, 'password: ' . ( $this->_mask_passwords_in_log ? md5($password) : $password ) );
		
		// Load options from WordPress-DB.
		$this->_load_options();
		
		// Log informations
		$this->_log( ADAI_LOG_INFO, "Options for adLDAP connection:\n" .
					  "- account_suffix: $this->user_account_suffix\n" .					
					  "- base_dn: $this->base_dn\n" .
					  "- domain_controllers: $this->domain_controllers\n" .
					  "- ad_username: $this->bind_user\n" .
					  "- ad_password: " . ( $this->_mask_passwords_in_log ? md5($this->bind_user_password) : $this->bind_user_password ). "\n" .
					  "- ad_port: $this->port\n" .
					  "- use_tls: $this->secure_connection" );

		$LDAP_Options = array(
						"base_dn"				=> $this->base_dn, 
						"domain_controllers"	=> explode( ';', $this->domain_controllers ),
						"ad_username"			=> $this->bind_user,			// AD Bind User
						"ad_password"			=> $this->bind_user_password,	// password
						"ad_port"				=> $this->port,					// AD port
						"use_tls"				=> $this->secure_connection		// secure?
						);
		if( $this->append_ad_user_suffix )
			$LDAP_Options['account_suffix'] = $this->ad_account_suffix;
		else
			$LDAP_Options['account_suffix'] = '';
			
		// Connect to Active Directory
		try {
			$this->_adldap = @new adLDAP( $LDAP_Options );
		} catch ( Exception $e ) {
    		$this->_log( ADAI_LOG_ERROR, 'adLDAP exception: ' . $e->getMessage() );
			$this->_log_flush();
    		return false;
		}
					
		$this->_log( ADAI_LOG_NOTICE, 'adLDAP object created.' );
		ob_start();
		var_dump( $this->_adldap );
		$this->_log( ADAI_LOG_INFO, ob_get_contents() );
		ob_end_clean();
		
		// Check for maximum login attempts
		$this->_log( ADAI_LOG_INFO, 'max_login_attempts: ' . $this->max_login_attempts );
		if ($this->max_login_attempts > 0) {
			$failed_logins = $this->_get_failed_logins_within_block_time($username);
			$this->_log(ADAI_LOG_INFO,'users failed logins: '.$failed_logins);
			if ($failed_logins >= $this->max_login_attempts) {
				$this->_authenticated = false;

				$this->_log(ADAI_LOG_ERROR,'Authentication failed');
				$this->_log(ADAI_LOG_ERROR,"Account '$username' blocked for $this->blocking_time seconds");
				
				// e-mail notfications if user is blocked
				if ($this->notify_user) {
					$this->_notify_user($username);
					$this->_log(ADAI_LOG_NOTICE,"Notification sent to user.");
				}
				if ($this->notify_admin) {
					$this->_notify_admin($username);
					$this->_log(ADAI_LOG_NOTICE,"Notification sent to admin(s).");
				}
				
				// Show the blocking page to the user (only if we are not in debug/log mode)
				if ($this->_loglevel == ADAI_LOG_NONE) {
					$this->_display_blocking_page($username);
				}
				die(); // important !
			} 
		}
		
		// This is where the action is.
		if ( $this->_adldap->authenticate($username, $password) )
		{	
			$this->_log(ADAI_LOG_NOTICE,'Authentication successfull');
			$this->_authenticated = true;
		}

		if ( $this->_authenticated == false )
		{
			$this->_log(ADAI_LOG_ERROR,'Authentication failed');
			$this->_authenticated = false;
			$this->_store_failed_login($username);
			$this->_log_flush();
			return false;			
		}
		
		// Cleanup old database entries 
		$this->_cleanup_failed_logins($username);

		// Check the authorization
		if ($this->auth_from_ad_grp) {
			if ($this->_check_authorization_by_group($username)) {
				$this->_authenticated = true;
			} else {
				$this->_authenticated = false;
				$this->_log_flush();
				return false;	
			}
		}
		
		$ad_username = $username;
		
		$user = get_userdatabylogin($username);
		if( !$user && $this->append_user_suffix ) {
			$username .= $this->user_account_suffix;
			$user = get_userdatabylogin( $username );
		}
		
		if (!$user OR ($user->user_login != $username)) {
			/*$this->_log( ADAI_LOG_FATAL, 'The user login ' . $user->user_login . ' did not match the passed username ' . $username );
			$this->_log_flush();
			die();*/
			$user_role = $this->_get_user_role_equiv($ad_username);
			if ($this->auto_user_create || trim($user_role) != '' ) {
					// create user
					$userinfo = $this->_adldap->user_info($ad_username, array('sn', 'givenname', 'mail', 'displayName', 'description', 'cn'));
					$userinfo = $userinfo[0];
					$email = $userinfo['mail'][0];
					$first_name = $userinfo['givenname'][0];
					$last_name = $userinfo['sn'][0];
					$display_name = $this->_get_display_name_from_AD($username, $userinfo);
					$user_id = $this->_create_user($ad_username, $email, $first_name, $last_name, $display_name, $user_role);
			} else {
				// Bail out to avoid showing the login form
				$this->_log(ADAI_LOG_ERROR,'This user exists in Active Directory, but has not been granted access to this installation of WordPress.');
				$this->_log_flush();
				return new WP_Error('invalid_username', __('<strong>ERROR</strong>: This user exists in Active Directory, but has not been granted access to this installation of WordPress.'));
			}
		} else {
			
			//  update known users if configured
			if ( $this->auto_user_create && $this->auto_user_update ) {
				/*$this->_log( ADAI_LOG_FATAL, 'We are trying to login to the blog with an ID of ' . $GLOBALS['blog_id'] );
				$this->_log_flush();
				die();*/
				
				if( $this->auto_update_user_group || ( is_multisite() && !is_user_member_of_blog( $user->ID, $GLOBALS['blog_id'] ) ) ) {
					$this->_log( ADAI_LOG_DEBUG, 'Either the auto_update_user_group option is set to true, or the user ' . $ad_username . ' is not currently a member of the blog with an ID of ' . $GLOBALS['blog_id'] . '. Therefore, the user role is being set based on the equivalent AD user group.' );
					// Update users role
					$user_role = $this->_get_user_role_equiv($ad_username);
				} else {
					$this->_log( ADAI_LOG_DEBUG, 'The auto_update_user_group option is disabled, and the user ' . $ad_username . ' appears to be a member of the blog with an ID of ' . $GLOBALS['blog_id'] . '. Therefore, no changes were made to this user\'s role.' );
					$user_role = '';
				}
				$userinfo = $this->_adldap->user_info($ad_username, array('sn', 'givenname', 'mail', 'displayName', 'description', 'cn'));
				$userinfo = $userinfo[0];
				
				$email = $userinfo['mail'][0];
				$first_name = $userinfo['givenname'][0];
				$last_name = $userinfo['sn'][0];
				$common_name = $userinfo['cn'][0];
				$display_name = $this->_get_display_name_from_AD($username, $userinfo);
				$user_id = $this->_update_user($ad_username, $email, $first_name, $last_name, $display_name, $user_role);
			}
		}
		
		// load user object
		if (!$user_id) {
			require_once( ABSPATH . WPINC . DIRECTORY_SEPARATOR . 'registration.php' ); 
			$user_id = username_exists($username);
			$this->_log(ADAI_LOG_NOTICE,'user_id: '.$user_id);
		}
		$user = new WP_User($user_id);

		$this->_log(ADAI_LOG_NOTICE,'FINISHED');
		$this->_log_flush();
		return $user;
	} /* AD_Authenticate function */

	/**
	 * Determines the number of failed login attempts of specific user within a specific time from now to the past.
	 * 
	 * @param $username
	 * @param $seconds number of seconds
	 * @return number of failed login attempts  
	 *
	 * @uses ADAuthInt_Plugin::get_table_name()
	 * @uses ADAuthInt_Plugin::TABLE_NAME
	 * @uses $wpdb
	 */
	protected function _get_failed_logins_within_block_time($username) {
		global $wpdb;
		$table_name = $this->get_table_name();
		$time = time() - (int)$this->blocking_time;
		
		$sql = "SELECT count(*) AS count from $table_name WHERE user_login = '" . $wpdb->escape( $username ) . "' AND failed_login_time >= $time";
		return $wpdb->get_var( $sql );
	} /* _get_failed_logins_within_block_time function */

	/**
	 * Send an email to the user who's account is blocked
	 * 
	 * @param $username string
	 * @return unknown_type
	 */
	protected function _notify_user($username)
	{
		// if auto creation is enabled look for the user in AD 
		if ($this->auto_user_create) {
			
			$userinfo = $this->_adldap->user_info($username, array("sn", "givenname", "mail"));
			if ($userinfo) {
				$userinfo = $userinfo[0];
				$email = $userinfo['mail'][0];
				$first_name = $userinfo['givenname'][0];
				$last_name = $userinfo['sn'][0];	
			} else { 
				return false;
			}
		} else {
			// auto creation is disabled, so look for the user in local database
			require_once(ABSPATH . WPINC . DIRECTORY_SEPARATOR . 'registration.php');
			$user_id = username_exists($username);
			if ($user_id) {
				$user_info = get_userdata($user_id);
				$last_name = $user_info->last_name;
				$first_name = $user_info->first_name;
				$email = $user_info->user_email;
			} else {
				return false;
			}
		}

		// do we have a correct email address?
		if (is_email($email)) { 
			$blog_url = get_bloginfo('url');
			$blog_name = get_bloginfo('name');
			$blog_domain = preg_replace ('/^(http(s*):\/\/)(.+)\/.*$/i','$3', $blog_url);
			

			$subject = '['.$blog_name.'] ' . __( 'Account blocked', ADAUTHINT_TEXT_DOMAIN );
			$body = sprintf( __( 'Someone tried to login to %s (%s) with your username (%s) - but in vain. For security reasons your account is now blocked for %d seconds.', ADAUTHINT_TEXT_DOMAIN ), $blog_name, $blog_url, $username, $this->blocking_time );
			$body .= "\n\r";
			$body .= __( 'THIS IS A SYSTEM GENERATED E-MAIL, PLEASE DO NOT RESPOND TO THE E-MAIL ADDRESS SPECIFIED ABOVE.', ADAUTHINT_TEXT_DOMAIN );
			
			/*$header = 'From: "WordPress" <wordpress@'.$blog_domain.">\r\n";*/
			return wp_mail($email, $subject, $body, $header);
		} else {
			return false;
		}
	}

	/**
	 * Notify administrator(s) by e-mail if an account is blocked
	 * 
	 * @param $username username of the blocked account
	 * @return boolean false if no e-mail is sent, true on success
	 */
	protected function _notify_admin($username)
	{
		$arrEmail = array(); // list of recipients
		
		if ($this->notify_admin) {
			$email = $this->admin_email;
			
			// Should we use Blog-Administrator's e-mail
			if (trim($email) == '') {
				// Is this an e-mail address?
				if (is_email($email)) {
					$arrEmail[0] = trim( get_bloginfo( 'admin_email' ) );
				}
			} else {
				// Using own list of notification recipients
				$arrEmail = explode(";",$email);
				
				// remove wrong e-mail addresses from array
				for ($x=0; $x < count($arrEmail); $x++) {
					$arrEmail[$x] = trim($arrEmail[$x]); // remove possible whitespaces
					if (!is_email($arrEmail[$x])) {
						unset($arrEmail[$x]);
					}
				}
				
			}
			
			// Do we have valid e-mail addresses?
			if (count($arrEmail) > 0) {
				
				if ($this->auto_user_create) {

					// auto creation is enabled, so look for the user in AD						
					$userinfo = $this->_adldap->user_info($username, array("sn", "givenname", "mail"));
					if ($userinfo) {
						$userinfo = $userinfo[0];
						$first_name = $userinfo['givenname'][0];
						$last_name = $userinfo['sn'][0];	
					} else { 
						return false;
					}
				} else {
					
					// auto creation is disabled, so look for the user in local database
					require_once(ABSPATH . WPINC . DIRECTORY_SEPARATOR . 'registration.php');
					$user_id = username_exists($username);
					if ($user_id) {
						$user_info = get_userdata($user_id);
						$last_name = $user_info->last_name;
						$first_name = $user_info->first_name;
					} else {
						return false;
					}
				}
			
				$blog_url = get_bloginfo('url');
				$blog_name = get_bloginfo('name');
				$blog_domain = preg_replace ('/^(http(s*):\/\/)(.+)\/.*$/i','$3', $blog_url);

				$subject = '['.$blog_name.'] ' . __( 'Account blocked', ADAUTHINT_TEXT_DOMAIN );
				$body = sprintf( __( 'Someone tried to login to %s (%s) with the username "%s" (%s %s) - but in vain. For security reasons this account is now blocked for %d seconds.', ADAUTHINT_TEXT_DOMAIN ), $blog_name, $blog_url, $username, $first_name, $last_name, $this->blocking_time );
				$body .= "\n\r";
				$body .= sprintf( __( 'The login attempt was made from IP-Address: %s', ADAUTHINT_TEXT_DOMAIN ), $_SERVER['REMOTE_ADDR']);
				$body .= "\n\r";
				$body .= __( 'THIS IS A SYSTEM GENERATED E-MAIL, PLEASE DO NOT RESPOND TO THE E-MAIL ADDRESS SPECIFIED ABOVE.', ADAUTHINT_TEXT_DOMAIN );
				/*$header = 'From: "WordPress" <wordpress@'.$blog_domain.">\r\n";*/
				
			
				// send e-mails
				$blnSuccess = true;
				foreach($arrEmail AS $email)  {
					$blnSuccess = ($blnSuccess AND wp_mail($email, $subject, $body, $header));
				}
				return $blnSuccess;
				
				
			} else {
				return false;
			}
		} else {
			return false;
		}
		
		return true;
	} 
	
	/**
	 * Output debug informations
	 * 
	 * @param integer level
	 * @param string $notice
	 */
	protected function _log($level = 0, $info = '') {
		if ($level <= $this->_loglevel) {
			$this->_debug .= '[' .$level . '] '.$info."\n\r";
		}
	} /* _log function */
	
	/**
	 * Output debug information and clean the log
	 */
	protected function _log_flush() {
		echo $this->_debug;
		$this->_debug = '';
	}

	/**
	 * Show a blocking page for blocked accounts.
	 * 
	 * @param $username
	 * @uses ADAuthInt_Plugin::_get_rest_of_blocking_time()
	 */
	protected function _display_blocking_page($username) {
		$seconds = $this->_get_rest_of_blocking_time($username);
		$output = '
<div id="login">
	<h1>
    	<a href="' . get_bloginfo( 'url' ) . '" title="' . apply_filters( 'login_headertitle', __('Powered by WordPress') ) . '">
			' . get_bloginfo('name') . '
        </a>
    </h1>
	<div id="login_error">
		' . sprintf( __( 'Account blocked for <span id="secondsleft">%d</span> seconds.', ADAUTHINT_TEXT_DOMAIN ), $seconds ) . '
	</div>
</div>';
		wp_die( $output );
	} /* _display_blocking_page */
	
	/**
	 * Get the rest of the time an account is blocked. 
	 * 
	 * @param $username
	 * @return int seconds the account is blocked, or 0
	 * @uses ADAuthInt_Plugin::get_table_name()
	 * @uses $wpdb::get_var()
	 * @uses ADAuthInt_Plugin::blocking_time
	 */
	protected function _get_rest_of_blocking_time($username) {
		global $wpdb;
		
		$table_name = $this->get_table_name();
		
		$sql = "SELECT max(failed_login_time) FROM $table_name WHERE user_login = '" . $wpdb->escape( $username ) . "'";
		$max_time = $wpdb->get_var($sql);
		
		if ($max_time == NULL ) {
			return 0;
		}
		return ( $max_time + $this->blocking_time ) - time();
	} /* _get_rest_of_blocking_time function */

	/**
	 * Stores the username and the current time in the db.
	 * 
	 * @param $username
	 * @return unknown_type
	 * @uses ADAuthInt_Plugin::get_table_name()
	 * @uses $wpdb
	 */
	protected function _store_failed_login($username) {
		global $wpdb, $site_id;
		
		$this->_log( ADAI_LOG_WARN, 'storing failed login for user "' . $username . '"' );
		$table_name = $this->get_table_name();
		
		$sql = "INSERT INTO $table_name ( user_login, failed_login_time, site_id ) VALUES ('" . $wpdb->escape( $username )."'," . time() . ", '" . $site_id . "' )";
		$result = $wpdb->query( $sql );
	} /* _store_failed_login function */

	/**
	 * Deletes entries from store where the time of failed logins is more than the specified block time ago.
	 * Deletes also all entries of a user, if its username is given . 
	 *  
	 * @param $username
	 * @return 
	 * @uses ADAuthInt_Plugin::get_table_name()
	 * @uses ADAuthInt_Plugin::_log()
	 */
	protected function _cleanup_failed_logins($username = NULL) {
		global $wpdb;
		
		$this->_log( ADAI_LOG_NOTICE, 'cleaning up failed logins for user "' . $username . '"' );
		$table_name = $this->get_table_name();
		$time = time() - $this->blocking_time;
		
		$sql = "DELETE FROM $table_name WHERE failed_login_time < $time";
		if ($username != NULL) {
			$sql .= " OR user_login = '".$wpdb->escape($username)."'"; 
		}
		
		$results = $wpdb->query($sql);
	} /* _cleanup_failed_logins function */

	/**
	 * Checks if the user is member of the group(s) allowed to login
	 * 
	 * @param $username
	 * @return boolean
	 *
	 * @uses ADAuthInt_Plugin::auth_from_ad_grp
	 * @uses ADAuthInt_Plugin::auth_groups
	 * @uses ADAuthInt_Plugin::_adldap
	 * @uses ADAuthInt_Plugin::_log()
	 */
	protected function _check_authorization_by_group($username) {
		if (!$this->auth_from_ad_grp) {
			return true;
		} else {
			$authorization_groups = explode(';', $this->auth_groups);
			foreach ($authorization_groups as $authorization_group) {
				if ($this->_adldap->user_ingroup($username, $authorization_group, true)) {
					$this->_log(ADAI_LOG_NOTICE,'Authorized by membership of group "'.$authorization_group.'"');
					return true;
				}
			}
			$this->_log( ADAI_LOG_WARN, 'Authorization by group failed. User is not authorized.' );
			return false;
		}
	} /* _check_authorization_by_group function */

	/**
	 * Get the first matching role from the list of role equivalent groups the user belongs to.
	 * 
	 * @param $ad_username 
	 * @return string matching role
	 *
	 * @uses ADAuthInt_Plugin::role_equiv_groups
	 * @uses ADAuthInt_Plugin::_adldap
	 * @uses ADAuthInt_Plugin::_log
	 * @uses ADAI_LOG_INFO
	 */
	protected function _get_user_role_equiv($ad_username) {
		
		$role_equiv_groups = explode(';', $this->role_equiv_groups);
		
		$user_role = '';
		foreach ($role_equiv_groups as $whatever => $role_group)
		{
				$role_group = explode('=', $role_group);
				if ( count($role_group) != 2 )
				{
					continue;
				}
				$ad_group = $role_group[0];
				
				$corresponding_role = $role_group[1];
				if ( $this->_adldap->user_ingroup($ad_username, $ad_group, true ) )
				{
					$user_role = $corresponding_role;
					break;
				}
		}
		$this->_log(ADAI_LOG_INFO,'user role: '.$user_role);
		return $user_role;
	} /* _get_user_role_equiv function */

	/**
	 * Determine the display_name to be stored in WP database.
	 * @param $username  the username used to login
	 * @param $userinfo  the array with data returned from AD
	 * @return string  display_name
	 * @uses ADAuthInt_Plugin::display_name
	 */
	protected function _get_display_name_from_AD($username, $userinfo) {
		if ( ( $this->display_name == '' ) || ( $this->display_name == 'sAMAccountName' ) ) {
			return $username;
		}
		$display_name = ( array_key_exists( $this->display_name, $userinfo ) ) ? 
			$userinfo[$this->display_name][0] : 
			'';
		return ( $display_name == '' ) ?
			$username :
			$display_name;
	} /* _get_display_name_from_AD function */

	/**
	 * Create a new WordPress account for the specified username.
	 * @param $username
	 * @param $email
	 * @param $first_name
	 * @param $last_name
	 * @param $display_name
	 * @param $role
	 * @return integer user_id
	 *
	 * @uses ADAuthInt_Plugin::_get_password()
	 * @uses ADAuthInt_Plugin::_create_non_duplicate_email()
	 * @uses ADAuthInt_Plugin::_log()
	 *
	 * @uses ADAuthInt_Plugin::default_email_domain
	 * @uses ADAuthInt_Plugin::append_user_suffix
	 * @uses ADAuthInt_Plugin::user_account_suffix
	 * @uses ADAuthInt_Plugin::dup_account_handling
	 *
	 * / WordPress Constants /
	 * @uses ABSPATH
	 * @uses WPINC
	 * @uses DIRECTORY_SEPARATOR
	 *
	 * / WordPress Functions /
	 * @uses wp_create_user()
	 * @uses is_wp_error()
	 * @uses username_exists()
	 * @uses update_user_meta()
	 * @uses update_usermeta()
	 * @uses wp_update_user()
	 */
	protected function _create_user($username, $email, $first_name, $last_name, $display_name = '', $role = '') {
		global $wp_version;
		
		$password = $this->_get_password();
		
		if ( $email == '' ) 
		{
			if (trim($this->default_email_domain) != '') {
				$email = $username . '@' . $this->default_email_domain;
			} else {
				if (strpos($username, '@') !== false) {
					$email = $username;
				}
			}
		}
				
		// append account suffix to new users? 
		if ($this->append_user_suffix) {
			$username .= $this->user_account_suffix;
		}
		
		$this->_log( ADAI_LOG_NOTICE, "Creating user '$username' with following data:\n".
					  "- email: $email\n".
					  "- first name: $first_name\n".
					  "- last name: $last_name\n".
					  "- display name: $display_name\n".
					  "- role: $role" );
		
		require_once( ABSPATH . WPINC . DIRECTORY_SEPARATOR . 'registration.php' );
		
		if ( $this->dup_account_handling == 'allow' ) {
			if ( !defined('WP_IMPORTING') ) {
				define( 'WP_IMPORTING', true ); // This is a dirty hack. See wp-includes/registration.php
			}
		}
		
		if ( $this->dup_account_handling == 'create' ) {
			$new_email = $this->_create_non_duplicate_email($email);
			if ($new_email !== $email) {
				$this->_log( ADAI_LOG_NOTICE, "Duplicate email address prevention: Email changed from $email to $new_email." );
			}
			$email = $new_email;
		}
		
		$email = trim( $email );
		// Here we go!
		$return = empty( $email ) ? wp_create_user( $username, $password ) : wp_create_user( $username, $password, $email );

		// log errors
		if ( is_wp_error( $return ) ) {
   			$this->_log( ADAI_LOG_ERROR, $return->get_error_message() );
		}
		
		$user_id = username_exists( $username );
		$this->_log( ADAI_LOG_NOTICE, ' - user_id: ' . $user_id );
		if ( !$user_id ) {
			$this->_log( ADAI_LOG_FATAL, 'Error creating user.' );
			wp_die( '<p>' . __( 'Error creating user!', ADAUTHINT_TEXT_DOMAIN ) . '</p>' . ( isset( $errorlog ) ?  '<p>' . $errorlog . '</p>' : '' ) . $this->_log_flush() );
			return;
		} else {
			if ( version_compare( $wp_version, '3', '>=' ) ) {
				// WP 3.0 and above
				update_user_meta( $user_id, 'first_name', $first_name );
				update_user_meta( $user_id, 'last_name', $last_name );
			} else {
				// WP 2.x
				update_usermeta( $user_id, 'first_name', $first_name );
				update_usermeta( $user_id, 'last_name', $last_name );
			}
			
			// set display_name
			if ( !empty( $display_name ) ) {
				$return = wp_update_user( array(
					'ID' => $user_id, 
					'display_name' => $display_name
				) );
			}
			
			// set role
			if ( !empty( $role ) ) {
				$return = wp_update_user( array( 
					'ID' => $user_id, 
					'role' => $role
				) );
			}
		}
		
		return $user_id;
	}
	
	/**
	 * Returns the given email address or a newly created so no 2 users
	 * can have the same email address.
	 * 
	 * @param $email original email address
	 * @return unique email address
	 */
	protected function _create_non_duplicate_email( $email ) {
		if ( !email_exists( $email ) ) {
			return $email;
		}
		
		// Ok, lets create a new email address that does not already exists in the database
		$arrEmailParts = split( '@', $email );
		$counter = 1;
		$ok = false;
		while ( $ok !== true ) {
			$email = $arrEmailParts[0] . $counter . '@' . $arrEmailParts[1];
			$ok = !email_exists( $email );
			$counter++;	
		}
		return $email;
	}
	
	/**
	 * Updates a specific Wordpress user account
	 * 
	 * @param $username
	 * @param $email
	 * @param $first_name
	 * @param $last_name
	 * @param $display_name
	 * @param $role
	 * @return integer user_id
	 *
	 * @uses $wp_version
	 * @uses ADAuthInt_Plugin::default_email_domain
	 * @uses ADAuthInt_Plugin::append_user_suffix
	 * @uses ADAuthInt_Plugin::user_account_suffix
	 * @uses ADAuthInt_Plugin::_log()
	 * @uses ADAI_LOG_NOTICE
	 * @uses username_exists()
	 * @uses ADAI_LOG_FATAL
	 * @uses update_user_meta
	 * @uses update_usermeta
	 * @uses wp_update_user
	 * @uses is_wp_error()
	 */
	protected function _update_user( $username, $email, $first_name, $last_name, $display_name = '', $role = '' ) {
		global $wp_version;
		
		if ( $email == '' ) {
			if ( trim( $this->default_email_domain ) != '' )
				$email = $username . '@' . $this->default_email_domain;
			elseif ( strpos( $username, '@' ) !== false )
				$email = $username;
		}
		
		$user_id = username_exists( $username );
		if( !$user_id && $this->append_user_suffix ) {
			$username .= $this->user_account_suffix;
			$user_id = username_exists( $username );
		}
		
		$this->_log(ADAI_LOG_NOTICE,'Updating user "'.$username."\" with following data:\n".
					  "- email: $email\n".
					  "- first name: $first_name\n".
					  "- last name: $last_name\n".
					  "- display name: $display_name\n".
					  "- role: $role");
		
		require_once(ABSPATH . WPINC . DIRECTORY_SEPARATOR . 'registration.php');
		
		$this->_log(ADAI_LOG_NOTICE,' - user_id: '.$user_id);
		if ( !$user_id ) {
			$this->_log(ADAI_LOG_FATAL,'Error updating user.');
			wp_die('<p>' . __( 'Error updating user!', ADAUTHINT_TEXT_DOMAIN ) . '</p>' . '<p>' . sprintf( __( 'For some reason, a user with the username %s does not appear to exist in the WordPress database.', ADAUTHINT_TEXT_DOMAIN), $username ) . '</p>' . $this->_log_flush() );
			return;
		} else {
			if( is_multisite() && !is_user_member_of_blog( $user_id, $GLOBALS['blog_id'] ) ) {
				add_user_to_blog( $GLOBALS['blog_id'], $user_id, NULL );
			}
			if (version_compare($wp_version, '3', '>=')) {
				// WP 3.0 and above
				update_user_meta($user_id, 'first_name', $first_name);
				update_user_meta($user_id, 'last_name', $last_name);
			} else {
				// WP 2.x
				update_usermeta($user_id, 'first_name', $first_name);
				update_usermeta($user_id, 'last_name', $last_name);
			}
			
			// set display_name
			if ($display_name != '') {
				wp_update_user(array('ID' => $user_id, 'display_name' => $display_name));
			}
			
			// set role
			if ( $role != '' ) 
			{
				wp_update_user(array('ID' => $user_id, 'role' => $role));
			}
			
			// set email
			if ( $email != '' ) 
			{
				$return = wp_update_user(array('ID' => $user_id, 'user_email' => $email));
			}
		}
		
		// log errors
		if (isset($return)) {
			if (is_wp_error($return)) {
	   			$this->_log(ADAI_LOG_ERROR, $return->get_error_message());
			}
		}
		
		return $user_id;
	} /* _update_user function */
	
	/*
	 * Used to disable certain login functions, e.g. retrieving a
	 * user's password.
	 */
	public function disable_function() {
		die('Disabled');
	} /* disable_function function */

	/*
	 * Generate a password for the user. This plugin does not
	 * require the user to enter this value, but we want to set it
	 * to something nonobvious.
	 */
	public function generate_password($username, $password1, $password2) {
		if( !$this->append_user_suffix || stristr( $username, $this->user_account_suffix ) )
			$password1 = $password2 = $this->_get_password();
	} /* generate_password function */

	/*
	 * Used to disable certain display elements, e.g. password
	 * fields on profile screen.
	 */
	public function disable_password_fields($show_password_fields) {
		return false;
	} /* disable_password_fields function */
	
	/**
	 * Generate a random password.
	 * 
	 * @param int $length Length of the password
	 * @return password as string
	 */
	protected function _get_password($length = 10) {
		return substr(md5(uniqid(microtime())), 0, $length);
	}

	/****************************************************************
	 * STATIC FUNCTIONS
	 ****************************************************************/

	/**
	 * Determine global table prefix, usually "wp_".
	 * 
	 * @return string table prefix
	 */
	public static function get_table_name() {
		global $wpdb;
		return $wpdb->base_prefix . self::TABLE_NAME;
	} /* get_table_name function */

}
?>